==> http2-wrapper-0.4.0.tgz: package/source/client-request.js <==
'use strict';
const http2 = require('http2');
const {Writable} = require('stream');
const HTTP2IncomingMessage = require('./incoming-message');
const urlToOptions = require('./utils/url-to-options');
const proxyEvents = require('./utils/proxy-events');
const {
	ERR_HTTP_HEADERS_SENT,
	ERR_INVALID_HTTP_TOKEN,
	ERR_HTTP_INVALID_HEADER_VALUE,
	ERR_INVALID_ARG_TYPE,
	ERR_INVALID_PROTOCOL
} = require('./utils/errors');

const {
	HTTP2_HEADER_STATUS,
	HTTP2_HEADER_METHOD,
	HTTP2_HEADER_PATH,
	HTTP2_METHOD_CONNECT
} = http2.constants;

const kHeaders = Symbol('headers');
const kCustomSession = Symbol('customSession');
const kErrorHandler = Symbol('errorHandler');

const noop = () => {};

const allowedProtocols = new Set(['https:', 'http:']);

class HTTP2ClientRequest extends Writable {
	constructor(input, options, cb) {
		super();

		if (typeof input === 'string' || input instanceof URL) {
			// (input, options, cb)
			input = urlToOptions(new URL(input));
		}

		if (typeof options === 'function') {
			// (options, cb)
			cb = options;
			options = input;
		} else {
			options = {...input, ...options};
		}

		options.protocol = options.protocol || 'https:';
		options.port = options.port || (options.protocol === 'http:' ? 80 : 443);

		if (!allowedProtocols.has(options.protocol)) {
			throw new ERR_INVALID_PROTOCOL(options.protocol, 'https:');
		}

		this[kHeaders] = Object.create(null);

		this[kErrorHandler] = error => this.emit('error', error);

		// Don't confuse with the `net` module
		this.path = options.path || '/';
		this.session = options.session;
		options.path = options.socketPath;
		options.session = options.socketSession;

		if (this.session) {
			this[kCustomSession] = true;
		} else {
			this[kCustomSession] = false;
			this.session = http2.connect(options.authority || options, options);
		}

		// Forwards errors to this instance.
		this.session.once('error', this[kErrorHandler]);

		// Emits response `close` event
		this.session.once('close', () => {
			if (this.res) {
				if (this.res.readable) {
					this.res.once('end', () => this.res.emit('close'));
				} else {
					this.res.emit('close');
				}
			}
		});

		this.socket = this.session.socket;
		this.connection = this.socket;

		this.method = options.method || 'GET';

		if (cb) {
			this.once('response', cb);
		}

		this.res = null;
		this.aborted = false;

		try {
			if (options.headers) {
				for (const [header, value] of Object.entries(options.headers)) {
					this.setHeader(header, value);
				}
			}

			if (options.auth && !Reflect.has(this[kHeaders], 'authorization')) {
				this.setHeader('authorization', 'Basic ' + Buffer.from(options.auth).toString('base64'));
			}
		} catch (error) {
			if (!this[kCustomSession]) {
				this.session.close();
			}

			throw error;
		}

		if (options.timeout) {
			this.setTimeout(options.timeout);
		}

		// Move the `socket` event to next tick, so the user can
		// attach the listener before the event has been emitted.
		process.nextTick(() => this.emit('socket', this.socket));
	}

	_write(chunk, encoding, callback) {
		this.flushHeaders();
		this._request.write(chunk, encoding, callback);
	}

	_final(callback) {
		if (this.session.destroyed) {
			return;
		}

		this.flushHeaders();
		this._request.end(callback);
	}

	abort() {
		if (!this.aborted) {
			process.nextTick(self => self.emit('abort'), this);
		}
		this.aborted = true;

		if (this.res) {
			this.res._dump();
		}

		if (this._request) {
			this._request.close();
		} else if (!this[kCustomSession]) {
			this.session.close();
		}
	}

	_destroy(error) {
		if (this._request) {
			this._request.destroy(error);
		} else {
			if (!this[kCustomSession]) {
				this.session.close();
			}

			if (error) {
				process.nextTick(() => this.emit('error', error));
			}
		}
	}

	flushHeaders() {
		if (!this._request && !this.destroyed) {
			// Makes the real request.
			const headers = {
				[HTTP2_HEADER_METHOD]: this.method,
				...this[kHeaders]
			};

			// Update the `method` property in case user changed it
			this.method = headers[HTTP2_HEADER_METHOD].toUpperCase();
			const isConnectMethod = this.method === HTTP2_METHOD_CONNECT;

			// Must not specify the `:path` and `:scheme` headers for `CONNECT` requests.
			if (!isConnectMethod) {
				headers[HTTP2_HEADER_PATH] = this.path;
			}

			this._request = this.session.request(headers, {
				endStream: false
			});

			// Forwards `timeout`, `continue`, `close` and `error` events to this instance.
			if (!isConnectMethod) {
				proxyEvents(this._request, this, ['timeout', 'continue', 'close']);
				this._request.once('error', error => this.emit('error', Reflect.has(error, 'cause') ? error.cause : error));
			}

			// Detaches error handling (the `http2` module will pass them to the request instance).
			this.session.removeListener('error', this[kErrorHandler]);
			this.session.once('error', noop);

			// Clean up error handling. We're done.
			this._request.once('end', () => this.session.removeListener('error', noop));

			// This event tells we are ready to listen for the data.
			this._request.once('response', (headers, flags, rawHeaders) => {
				this.res = new HTTP2IncomingMessage(this.socket);
				this.res.req = this;
				this.res.statusCode = headers[HTTP2_HEADER_STATUS];
				this.res.headers = {...headers};
				this.res.rawHeaders = [...rawHeaders];

				this.res.once('end', () => {
					if (this.aborted) {
						this.res.aborted = true;
						this.res.emit('aborted');
					} else {
						this.res.complete = true;
					}
				});

				if (isConnectMethod) {
					this.res.upgrade = true;

					// The HTTP1 API says the socket is detached here,
					// but we can't do that so we pass the original HTTP2 request.
					if (this.emit('connect', this.res, this._request, Buffer.alloc(0))) {
						this.emit('close');
					} else {
						// No listeners attached, destroy the original request.
						this._request.destroy();
					}
				} else {
					// Forwards data
					this._request.pipe(this.res);

					if (!this.emit('response', this.res)) {
						// No listeners attached, dump the response.
						this.res._dump();
					}
				}
			});

			// Emits `information` event
			this._request.once('headers', headers => this.emit('information', {statusCode: headers[HTTP2_HEADER_STATUS]}));

			this._request.once('trailers', (trailers, flags, rawTrailers) => {
				// Assigns trailers to the response object.
				this.res.trailers = {...trailers};
				this.res.rawTrailers = [...rawTrailers];
			});

			if (!this[kCustomSession]) {
				// The session is ours, we won't reuse it.
				this._request.once('close', () => this.session.close());
			}

			this.emit('flushedHeaders');
		}
	}

	getHeader(name) {
		if (typeof name !== 'string') {
			throw new ERR_INVALID_ARG_TYPE('name', 'string', name);
		}

		return this[kHeaders][name.toLowerCase()];
	}

	get headersSent() {
		if (this._request) {
			return Boolean(this._request.sentHeaders);
		}

		return false;
	}

	removeHeader(name) {
		if (this.headersSent) {
			throw new ERR_HTTP_HEADERS_SENT('remove');
		}
		if (name === '' || typeof name !== 'string') {
			throw new ERR_INVALID_HTTP_TOKEN('Header name', name);
		}

		delete this[kHeaders][name.toLowerCase()];
	}

	setHeader(name, value) {
		if (this.headersSent) {
			throw new ERR_HTTP_HEADERS_SENT('set');
		}
		if (name === '' || typeof name !== 'string') {
			throw new ERR_INVALID_HTTP_TOKEN('Header name', name);
		}
		if (value === undefined) {
			throw new ERR_HTTP_INVALID_HEADER_VALUE(value, name);
		}

		this[kHeaders][name.toLowerCase()] = value;
	}

	setNoDelay() {
		// HTTP2 sockets cannot be malformed, do nothing.
	}

	setSocketKeepAlive() {
		// HTTP2 sockets cannot be malformed, do nothing.
	}

	setTimeout(ms, cb) {
		if (this._request) {
			this._request.setTimeout(ms, cb);
		} else {
			this.once('flushedHeaders', () => {
				this._request.setTimeout(ms, cb);
			});
		}

		return this;
	}
}

module.exports = HTTP2ClientRequest;

==> http2-wrapper-0.4.2.tgz: package/source/client-request.js <==
'use strict';
const http2 = require('http2');
const {Writable} = require('stream');
const HTTP2IncomingMessage = require('./incoming-message');
const urlToOptions = require('./utils/url-to-options');
const proxyEvents = require('./utils/proxy-events');
const {
	ERR_HTTP_HEADERS_SENT,
	ERR_INVALID_HTTP_TOKEN,
	ERR_HTTP_INVALID_HEADER_VALUE,
	ERR_INVALID_ARG_TYPE,
	ERR_INVALID_PROTOCOL
} = require('./utils/errors');

const {
	HTTP2_HEADER_STATUS,
	HTTP2_HEADER_METHOD,
	HTTP2_HEADER_PATH,
	HTTP2_METHOD_CONNECT
} = http2.constants;

const kHeaders = Symbol('headers');
const kCustomSession = Symbol('customSession');
const kErrorHandler = Symbol('errorHandler');

const noop = () => {};

class HTTP2ClientRequest extends Writable {
	constructor(input, options, cb) {
		super();

		if (typeof input === 'string' || input instanceof URL) {
			// (input, options, cb)
			input = urlToOptions(new URL(input));
		}

		if (typeof options === 'function') {
			// (options, cb)
			cb = options;
			options = input;
		} else {
			options = {...input, ...options};
		}

		options.protocol = options.protocol || 'https:';
		options.port = options.port || 443;

		if (options.protocol !== 'https:') {
			throw new ERR_INVALID_PROTOCOL(options.protocol, 'https:');
		}

		this[kHeaders] = Object.create(null);

		this[kErrorHandler] = error => this.emit('error', error);

		// Don't confuse with the `net` module.
		this.path = options.path || '/';
		this.session = options.session;
		options.path = options.socketPath;
		options.session = options.socketSession;

		if (this.session) {
			this[kCustomSession] = true;
		} else {
			this[kCustomSession] = false;
			this.session = http2.connect(options.authority || options, options);
		}

		// Forwards errors to this instance.
		this.session.once('error', this[kErrorHandler]);

		// Note: response `close` event is not supported,
		// because if you keep the session open till the process exits
		// there will be a memory leak.

		this.socket = this.session.socket;
		this.connection = this.socket;

		this.method = options.method || 'GET';

		if (cb) {
			this.once('response', cb);
		}

		this.res = null;
		this.aborted = false;

		try {
			if (options.headers) {
				for (const [header, value] of Object.entries(options.headers)) {
					this.setHeader(header, value);
				}
			}

			if (options.auth && !Reflect.has(this[kHeaders], 'authorization')) {
				this.setHeader('authorization', 'Basic ' + Buffer.from(options.auth).toString('base64'));
			}
		} catch (error) {
			if (!this[kCustomSession]) {
				this.session.close();
			}

			throw error;
		}

		if (options.timeout) {
			this.setTimeout(options.timeout);
		}

		// Move the `socket` event to next tick, so the user can
		// attach the listener before the event has been emitted.
		process.nextTick(() => this.emit('socket', this.socket));
	}

	_write(chunk, encoding, callback) {
		this.flushHeaders();
		this._request.write(chunk, encoding, callback);
	}

	_final(callback) {
		if (this.session.destroyed) {
			return;
		}

		this.flushHeaders();
		this._request.end(callback);
	}

	abort() {
		if (!this.aborted) {
			process.nextTick(self => self.emit('abort'), this);
		}
		this.aborted = true;

		if (this.res) {
			this.res._dump();
		}

		if (this._request) {
			this._request.close();
		} else if (!this[kCustomSession]) {
			this.session.close();
		}
	}

	_destroy(error) {
		if (this._request) {
			this._request.destroy(error);
		} else {
			if (!this[kCustomSession]) {
				this.session.close();
			}

			if (error) {
				process.nextTick(() => this.emit('error', error));
			}
		}
	}

	flushHeaders() {
		if (!this._request && !this.destroyed) {
			// Makes the real request.
			const headers = {
				[HTTP2_HEADER_METHOD]: this.method,
				...this[kHeaders]
			};

			// Update the `method` property in case user changed it.
			this.method = headers[HTTP2_HEADER_METHOD].toUpperCase();
			const isConnectMethod = this.method === HTTP2_METHOD_CONNECT;

			// Must not specify the `:path` and `:scheme` headers for `CONNECT` requests.
			if (!isConnectMethod) {
				headers[HTTP2_HEADER_PATH] = this.path;
			}

			this._request = this.session.request(headers, {
				endStream: false
			});

			// Forwards `timeout`, `continue`, `close` and `error` events to this instance.
			if (!isConnectMethod) {
				proxyEvents(this._request, this, ['timeout', 'continue', 'close']);
				this._request.once('error', error => this.emit('error', Reflect.has(error, 'cause') ? error.cause : error));
			}

			// Detaches error handling (the `http2` module will pass them to the request instance).
			this.session.removeListener('error', this[kErrorHandler]);
			this.session.once('error', noop);

			// Clean up error handling. We're done.
			this._request.once('end', () => this.session.removeListener('error', noop));

			// This event tells we are ready to listen for the data.
			this._request.once('response', (headers, flags, rawHeaders) => {
				this.res = new HTTP2IncomingMessage(this.socket);
				this.res.req = this;
				this.res.statusCode = headers[HTTP2_HEADER_STATUS];
				this.res.headers = {...headers};
				this.res.rawHeaders = [...rawHeaders];

				this.res.once('end', () => {
					if (this.aborted) {
						this.res.aborted = true;
						this.res.emit('aborted');
					} else {
						this.res.complete = true;
					}
				});

				if (isConnectMethod) {
					this.res.upgrade = true;

					// The HTTP1 API says the socket is detached here,
					// but we can't do that so we pass the original HTTP2 request.
					if (this.emit('connect', this.res, this._request, Buffer.alloc(0))) {
						this.emit('close');
					} else {
						// No listeners attached, destroy the original request.
						this._request.destroy();
					}
				} else {
					// Forwards data
					this._request.pipe(this.res);

					if (!this.emit('response', this.res)) {
						// No listeners attached, dump the response.
						this.res._dump();
					}
				}
			});

			// Emits `information` event
			this._request.once('headers', headers => this.emit('information', {statusCode: headers[HTTP2_HEADER_STATUS]}));

			this._request.once('trailers', (trailers, flags, rawTrailers) => {
				// Assigns trailers to the response object.
				this.res.trailers = {...trailers};
				this.res.rawTrailers = [...rawTrailers];
			});

			if (!this[kCustomSession]) {
				// The session is ours, we won't reuse it.
				this._request.once('close', () => this.session.close());
			}

			this.emit('flushedHeaders');
		}
	}

	getHeader(name) {
		if (typeof name !== 'string') {
			throw new ERR_INVALID_ARG_TYPE('name', 'string', name);
		}

		return this[kHeaders][name.toLowerCase()];
	}

	get headersSent() {
		if (this._request) {
			return Boolean(this._request.sentHeaders);
		}

		return false;
	}

	removeHeader(name) {
		if (this.headersSent) {
			throw new ERR_HTTP_HEADERS_SENT('remove');
		}
		if (name === '' || typeof name !== 'string') {
			throw new ERR_INVALID_HTTP_TOKEN('Header name', name);
		}

		delete this[kHeaders][name.toLowerCase()];
	}

	setHeader(name, value) {
		if (this.headersSent) {
			throw new ERR_HTTP_HEADERS_SENT('set');
		}
		if (name === '' || typeof name !== 'string') {
			throw new ERR_INVALID_HTTP_TOKEN('Header name', name);
		}
		if (value === undefined) {
			throw new ERR_HTTP_INVALID_HEADER_VALUE(value, name);
		}

		this[kHeaders][name.toLowerCase()] = value;
	}

	setNoDelay() {
		// HTTP2 sockets cannot be malformed, do nothing.
	}

	setSocketKeepAlive() {
		// HTTP2 sockets cannot be malformed, do nothing.
	}

	setTimeout(ms, cb) {
		if (this._request) {
			this._request.setTimeout(ms, cb);
		} else {
			this.once('flushedHeaders', () => {
				this._request.setTimeout(ms, cb);
			});
		}

		return this;
	}
}

module.exports = HTTP2ClientRequest;
